/*
Sector de arranque en GNU Assembler
Author: Erwin Meza Vega

Description:
Ver el archivo using_bios_services.txt para mas detalles acerca del
uso de los servicios de la BIOS.

Este sector de arranque ilustra el proceso para invocar algunos servicios
de la BIOS (Basic Input-Output System), entre ellos:

* Imprimir un caracter por pantalla
* Leer un caracter del teclado

El bootsector primero imprime la secuencia de caracteres "Hello\r\n", espera
a que se pulse una tecla y luego imprime la secuencia de caracteres "Bye".
*/

.intel_syntax noprefix /* Sintaxis Intel, evitar los prefijos en los registros*/
.code16     /* 16 bits- Modo Real */

.section .text  /* Seccion de texto */

.globl start

/* Simbolo de entrada del bootsector. En este punto comienza su ejecucion. */
start:

    /* Asegurarse que cs tome el valor de 0x7C0 e ip toma el valor del
    desplazamiento de la etiqueta 'entry_point' */
    ljmp 0x7C0 :  OFFSET entry_point

/* La ejecucion continua en este punto, pero se garantiza que el valor de cs
es 0x7C0*/
entry_point:

     /* Primero imprimir un mensaje por pantalla  usando los servicios de
     la BIOS */

     mov ah, 0x0E    /* Servicio de video a ejecutar (escribir caracter en
                           modo texto) */

     /* mov al, 0x48 */ /* 0x48 = caracter ASCII 'H'. GNU assembler
                              acepta las dos sintaxis. */
     mov al, 'H'     /* Caracter a imprimir.  */
     int 0x10          /* Invocar los servicios de video de la BIOS */

     mov al, 'e'     /* Repetir con los siguientes caracteres */
     int 0x10

     mov al, 'l'
     int 0x10

     mov al, 'l'
     int 0x10

     mov al, 'o'
     int 0x10

     /* mov al, 0x0D */ /* El caracter de Carriage Return es '\r' = 0x0D */
     mov al, '\r'
     int 0x10

      /* movb $0x0A, al */ /* El caracter de Line Feed es '\n' = 0x0A */
     mov al, '\n'
     int 0x10


     /* Ahora esperar por una tecla usando los servicios de la BIOS */

     /* mov ah, 0x00 */ /* mov reg, 0 es equivalente a xor reg, reg */
     xor ah, ah      /* Servicio a ejecutar: leer una tecla */
     int 0x16          /* Invocar los servicios de teclado de la BIOS */

    /* Ahora imprimir un mensaje de despedida */

     mov ah, 0x0E    /* De nuevo establecer ah para invocar el servicio de
                         video a ejecutar (escribir caracter en modo texto) */

     mov al, 'B'     /* Caracter a imprimir. 'B' */
     int 0x10

     mov al, 'y'
     int 0x10

     movb al, 'e'
     int 0x10

     /* Siempre al final el bootsector debe quedar en un ciclo infinito */

     jmp finished /* Saltar a el simbolo (la etiqueta) 'finished'


/* Simbolo (etiqueta) finished. */
finished:
   jmp finished   /* Salta a la etiqueta 'finished' (ciclo infinito) */

/*
Fin del bootsector
*/

/*

Una vez compilado, el codigo del bootsector no puede ocupar mas de 510 bytes.
Al generar el codigo ejecutable del bootsector, el linker se encarga
de rellenar con ceros el espacio a partir de la ultima instruccion de codigo
compilado hasta completar 510 bytes, y luego incluye los bytes 0x55 y 0xAA
para generar un bootsector valido de 512 bytes.

Ver el archivo link_bootsector.ld para mas detalles.

*/
