/*
Sector de arranque en GNU Assembler
Author: Erwin Meza Vega

Description:
Ver el archivo using_the_stack.txt para mas detalles acerca del
uso de la pila.

Este bootsector ilustra el uso de la pila del procesador Intel x86.
Para este proposito, almacena caracteres en la pila y luego los obtiene
de nuevo, para imprimirlos por pantalla haciendo uso de los servicios de la
BIOS.

*/

.intel_syntax noprefix /* Sintaxis Intel, evitar los prefijos en los registros*/
.code16     /* 16 bits- Modo Real */

.section .text  /* Seccion de texto */

.globl start

/* Simbolo de entrada del bootsector. En este punto comienza su ejecucion. */
start:

    /* Asegurarse que cs tome el valor de 0x7C0 e ip toma el valor del
    desplazamiento de la etiqueta 'entry_point' */
    ljmp 0x7C0 : OFFSET entry_point

/* La ejecucion continua en este punto, pero se garantiza que el valor de cs
es 0x7C0*/
entry_point:

    /* Primero se debe configurar la pila en una region de memoria
     que no este siendo utilizada. */

     /* Ver archivo real_mode_memory.txt para los detalles de como la BIOS
     configura el primer MegaByte de memoria en modo real. */

    cli     /* Deshabilitar las interrupciones */

    mov ax, 0x9000   /* La base de la pila se configura a 0x90000 */
    mov ss, ax       /* No se puede establecer directamente ss, usar ax*/
    mov sp, 0xFFFF   /* Configurar el tope de la pila a 0x90000 + 0xFFFF*/

    sti     /* Habilitar de nuevo las interrupciones */



    /* Imprimir el caracter 'A' = 0x41 usando los servicios de la BIOS */
    mov ah, 0x0E /* 0x0E = Servicio de la bios para escribir caracter */
    mov al, 'A'  /* Caracter a imprimir */
    int 0x10       /* Invocar los servicios de video de la BIOS */


    /* Almacenar ax (ah:al) en la pila. ah contiene el numero de servicio
    de la BIOS, y al contiene el caracter a imprimir.
    */

    push ax

    /* Sacar el valor del tope de la pila y almacenarlo en ax.
    ax deberia tomar el mismo valor que tenia antes: (ah = 0x0E,
    y al = 0x41 = 'A')  */

    pop ax
    int 0x10       /* Invocar los servicios de video de la BIOS */
                    /* Deberia imprimir de nuevo el mismo caracter */


    /* Almacenar todos los registros de proposito general */
    pusha

    /* Extraer 8 words de la pila y almacenarlos en los registros
    de proposito general */

    popa

    int 0x10       /* Invocar los servicios de video de la BIOS */
                    /* Deberia imprimir de nuevo el mismo caracter almacenado
                       en al, con los atributos almacenados en ah*/

    jmp finished /* Saltar a el simbolo (la etiqueta) 'finished'


/* Simbolo (etiqueta) finished. */
finished:
   jmp finished   /* Salta a la etiqueta 'finished' (ciclo infinito) */

/*
Fin del bootsector
*/

/*

Una vez compilado, el codigo del bootsector no puede ocupar mas de 510 bytes.
Al generar el codigo ejecutable del bootsector, el linker se encarga
de rellenar con ceros el espacio a partir de la ultima instruccion de codigo
compilado hasta completar 510 bytes, y luego incluye los bytes 0x55 y 0xAA
para generar un bootsector valido de 512 bytes.

Ver el archivo link_bootsector.ld para mas detalles.

*/
